### Yksinkertainen Vipusen rajapinnasta tietoa csv-tiedostoon noutava koodaus-Tiedostopolkuesimerkki
### 27.9.2022 CSC
### Huom. kaikki  ### merkit sisältävät rivit ovat selityksiä varten, joten ne voi poistaa tiedostosta huoletta

### VALMISTELUT
### haetaan tarvittavat kirjastot
import requests, time, sys
from pandas import DataFrame

### asetetaan pohjatietojatietoja
limit=5000
tietojoukko="julkaisut" # muiden tietojoukkojen nimet löytyvät rajapinnasta: https://api.vipunen.fi/api/resources/
tilastovuosi="2020"
max_latausyritykset = 3

### pohja rajapinnan urlille, johon myöhemmin lisätään offset arvo dynaamisesti. Limit-muuttuja on kokonaisluku (integer) joten se muunnetaan tekstiksi
### lisätty parametri sort=(%2Bdefaultorder), jotta aineisto saadaan varmasti oikeassa järjestyksessä rajapinnasta, jos muitakin järjestämisehtoja haluaa käyttää
url_pohja="https://api.vipunen.fi/api/resources/"+tietojoukko+"/data?filter=tilastovuosi=="+tilastovuosi+"&sort=(%2Bdefaultorder)&limit="+str(limit)+"&offset="
### 5000  limit-arvolla 3. kierroksen osoite olisi siten https://api.vipunen.fi/api/resources/"+tietojoukko+"/data?limit=5000&offset=10000

### Määritetään polku rajapinnan maksimiobjektien lukumäärän tiedustelua varten
max_rivimaara_url="https://api.vipunen.fi/api/resources/"+tietojoukko+"/data/count?filter=tilastovuosi=="+tilastovuosi

### Dynaaminen tiedostopolkuesimerkki csv-tiedostolle Windows
### Voi määrittää myös ilman muuttujia esim. d:/temp/20210414_tilastovuosi.csv
tiedosto="d:/temp/"+tietojoukko+"_"+tilastovuosi+".csv"

### Dynaaminen tiedostopolkuesimerkki csv-tiedostolle Linux / Mac
#tiedosto="/temp/"+tietojoukko+"_"+tilastovuosi+".csv"

### Asetetaan rajapintakyselyä varten http-otsikkotiedot
otsikkotiedot = {"Content-Type": "application/json"}
otsikkotiedot["Accept"] = "application/json"
### Hyvä käytäntö on kertoa mikä organisaatio tai tietoja noutaa esim. "1.2.246.562.10.2013112012294919827487.Vipunen"
otsikkotiedot["Caller-Id"] = "organisaatio_oid.organisaationimi"

### Haetaan maksimirivimäärä max_rivimaara-muuttujaan, jota hyödynnetään myöhemmin
max_rivimaara=requests.get(max_rivimaara_url,headers=otsikkotiedot).json()

### Määritetään laskurin alkuarvo, jota käytetään offset-arvon määrittämiseen
laskuri=0

###############################################
### Varsinainen rajapintakysely alkaa tästä ###
###############################################

### Haetaan rajapinnasta aineistoa, kunnes saadaan kaikki talletettua tiedostoon
while laskuri <=max_rivimaara:

    ### Alustetaan ja nollataan apidata, johon rajapinnasta noudettu data väliaikaisesti talletetaan
    apidata=[]

    ### Muodostetaan API-pyynnön dynaaminen osoite. laskuri-muuttuja on kokonaisluku (integer) joten se muunnetaan tekstiksi
    url=url_pohja+str(laskuri)

    ### Virheenhallinnan alustus
    latausyritys = 0
    lataustila="lataa"
    
    ### API-rajapintakutsu, jonka vastaus talletetaan muuttujaan vastaus
    ### 
    while latausyritys < max_latausyritykset and lataustila=="lataa":
        try:
            vastaus = requests.get(url, headers=otsikkotiedot).json()
            print(".")
            lataustila="valmis"
        except ValueError:
            latausyritys=+1
            print(latausyritys, ". uusi latausyritys")
            time.sleep(1)
            ### API-rajapintakutsu, jonka vastaus talletetaan muuttujaan vastaus
    if latausyritys == max_latausyritykset and lataustila=="lataa":
        print("Lataus epäonnistui. Offset-parametrin arvo oli:", laskuri)
        sys.exit(0)
        
    ### Talletetaan aineisto DataFrame muotoiseksi helpon jatkokäsittelyn mahdollistamiseksi
    apidata=DataFrame(vastaus)

    ### Write_mode-muuttujalla määritetään, että 1. kierroksella luodaan uusi tiedosto tai
    ### olemassa olemassaoleva ylikirjoitetaan. Jatkokierroksilla dataa vain lisätään tiedostoon
    ### Otsikkorivi kirjoitetaan vain ensimäisellä kierroksella
    if laskuri == 0:
        write_mode="w" ### write
        otsikkorivi=True
    else:
        write_mode="a" ### append
        otsikkorivi=False
    ### Data kirjoitetaan tai lisätään csv-tiedostoon jokaisella kyselykierroksella, ettei kaikkea haettua dataa käsitellä koneen muistissa.
    ### HUOM! Joidenkin rajapintojen datasta muodostetut tiedostot voivat kasvaa erittäin suuriksi
    ### Tiedoston asetusten määrittely. Lisätietoja löytyy internetistä esim. hakusanoilla "pandas" + "dataframe"

    apidata.to_csv(path_or_buf=tiedosto, sep=";", na_rep="",
                header=otsikkorivi, index=False, mode=write_mode, encoding="utf-8", quoting=0,
                quotechar='"', line_terminator="\n" , escapechar="$")

    ### Laskuriin lisätään haettavan tiedon määrä, jotta offset arvo muuttuu kun url seuraavan kerran määritetään luupissa
    laskuri=laskuri+limit

    ### Alla olevilla komennoilla seurataan latauksen edistymistä, joten ne voi automatisoinneista poistaa kokonaan
    if laskuri > max_rivimaara:
        print(max_rivimaara)
    else:
        print(laskuri)

### Tulostetaan loppuun vielä tiedoksi, että datan nouto on valmis
print("Valmis, ", max_rivimaara, " riviä haettu")
